@@ -1,3 +1,5 @@
+use crate::Error;
+
 pub struct MultiParams {
     min_size: usize,
     degree: i32,
@@ -41,13 +43,19 @@ impl MultiParams {
         self
     }
 
-    pub fn fit(&self, z: &[f64]) -> Vec<usize> {
-        assert!(self.min_size >= 2, "min_size must be at least 2");
-        assert!(self.beta.is_none() || self.percent.is_none(), "beta and percent cannot be passed together");
-        assert!(self.degree >= 0 && self.degree <= 2, "degree must be 0, 1, or 2");
+    pub fn fit(&self, z: &[f64]) -> Result<Vec<usize>, Error> {
+        if self.min_size < 2 {
+            return Err(Error::Parameter("min_size must be at least 2".to_string()));
+        }
+        if self.beta.is_some() && self.percent.is_some() {
+            return Err(Error::Parameter("beta and percent cannot be passed together".to_string()));
+        }
+        if self.degree < 0 || self.degree > 2 {
+            return Err(Error::Parameter("degree must be 0, 1, or 2".to_string()));
+        }
 
         if z.len() < self.min_size {
-            return Vec::new();
+            return Ok(Vec::new());
         }
 
         // scale observations
@@ -55,20 +63,22 @@ impl MultiParams {
         let max = z.iter().max_by(|i, j| i.partial_cmp(j).unwrap()).unwrap();
         let denom = max - min;
         if denom == 0.0 {
-            return Vec::new();
+            return Ok(Vec::new());
         }
         let zcounts: Vec<f64> = z.iter().map(|x| (x - min) / denom).collect();
 
         if self.percent.is_some() {
-            crate::edm_multi::edm_percent(&zcounts, self.min_size, self.percent.unwrap(), self.degree)
+            Ok(crate::edm_multi::edm_percent(&zcounts, self.min_size, self.percent.unwrap(), self.degree))
         } else {
-            crate::edm_multi::edm_multi(&zcounts, self.min_size, self.beta.unwrap_or(0.008), self.degree)
+            Ok(crate::edm_multi::edm_multi(&zcounts, self.min_size, self.beta.unwrap_or(0.008), self.degree))
         }
     }
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::Error;
+
     fn generate_series() -> Vec<f64> {
         vec![
             3.0, 1.0, 2.0, 3.0, 2.0, 1.0, 1.0, 2.0, 2.0, 3.0,
@@ -80,28 +90,28 @@ mod tests {
     #[test]
     fn test_multi() {
         let series = generate_series();
-        let breakouts = crate::multi().min_size(5).fit(&series);
+        let breakouts = crate::multi().min_size(5).fit(&series).unwrap();
         assert_eq!(vec![10, 15, 20], breakouts);
     }
 
     #[test]
     fn test_percent() {
         let series = generate_series();
-        let breakouts = crate::multi().min_size(5).percent(0.5).fit(&series);
+        let breakouts = crate::multi().min_size(5).percent(0.5).fit(&series).unwrap();
         assert_eq!(vec![8, 19], breakouts);
     }
 
     #[test]
     fn test_empty() {
         let series = Vec::new();
-        let breakouts = crate::multi().fit(&series);
+        let breakouts = crate::multi().fit(&series).unwrap();
         assert!(breakouts.is_empty());
     }
 
     #[test]
     fn test_constant() {
         let series = vec![1.0; 100];
-        let breakouts = crate::multi().fit(&series);
+        let breakouts = crate::multi().fit(&series).unwrap();
         assert!(breakouts.is_empty());
     }
 
@@ -109,7 +119,7 @@ mod tests {
     fn test_almost_constant() {
         let mut series = vec![1.0; 100];
         series[50] = 2.0;
-        let breakouts = crate::multi().fit(&series);
+        let breakouts = crate::multi().fit(&series).unwrap();
         assert!(breakouts.is_empty());
     }
 
@@ -119,28 +129,37 @@ mod tests {
             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
             1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0
         ];
-        let breakouts = crate::multi().min_size(5).fit(&series);
+        let breakouts = crate::multi().min_size(5).fit(&series).unwrap();
         assert_eq!(vec![10], breakouts);
     }
 
     #[test]
-    #[should_panic(expected = "min_size must be at least 2")]
     fn test_bad_min_size() {
         let series = Vec::new();
-        crate::multi().min_size(1).fit(&series);
+        let result = crate::multi().min_size(1).fit(&series);
+        assert_eq!(
+            result.unwrap_err(),
+            Error::Parameter("min_size must be at least 2".to_string())
+        );
     }
 
     #[test]
-    #[should_panic(expected = "beta and percent cannot be passed together")]
     fn test_beta_percent() {
         let series = Vec::new();
-        crate::multi().beta(0.008).percent(0.5).fit(&series);
+        let result = crate::multi().beta(0.008).percent(0.5).fit(&series);
+        assert_eq!(
+            result.unwrap_err(),
+            Error::Parameter("beta and percent cannot be passed together".to_string())
+        );
     }
 
     #[test]
-    #[should_panic(expected = "degree must be 0, 1, or 2")]
     fn test_bad_degree() {
         let series = Vec::new();
-        crate::multi().degree(3).fit(&series);
+        let result = crate::multi().degree(3).fit(&series);
+        assert_eq!(
+            result.unwrap_err(),
+            Error::Parameter("degree must be 0, 1, or 2".to_string())
+        );
     }
 }
